using ZhuyinPinyin

using Test

zhuyin_to_pinyin = Dict(
    "ㄚ" => "a",
    "ㄞ" => "ai",
    "ㄢ" => "an",
    "ㄤ" => "ang",
    "ㄠ" => "ao",
    "ㄅㄚ" => "ba",
    "ㄅㄞ" => "bai",
    "ㄅㄢ" => "ban",
    "ㄅㄤ" => "bang",
    "ㄅㄠ" => "bao",
    "ㄅㄟ" => "bei",
    "ㄅㄣ" => "ben",
    "ㄅㄥ" => "beng",
    "ㄅㄧ" => "bi",
    "ㄅㄧㄢ" => "bian",
    "ㄅㄧㄠ" => "biao",
    "ㄅㄧㄝ" => "bie",
    "ㄅㄧㄣ" => "bin",
    "ㄅㄧㄥ" => "bing",
    "ㄅㄛ" => "bo",
    "ㄅㄨ" => "bu",
    "ㄘㄚ" => "ca",
    "ㄘㄞ" => "cai",
    "ㄘㄢ" => "can",
    "ㄘㄤ" => "cang",
    "ㄘㄠ" => "cao",
    "ㄘㄜ" => "ce",
    "ㄘㄣ" => "cen",
    "ㄘㄥ" => "ceng",
    "ㄔㄚ" => "cha",
    "ㄔㄞ" => "chai",
    "ㄔㄢ" => "chan",
    "ㄔㄤ" => "chang",
    "ㄔㄠ" => "chao",
    "ㄔㄜ" => "che",
    "ㄔㄣ" => "chen",
    "ㄔㄥ" => "cheng",
    "ㄔ" => "chi",
    "ㄔㄨㄥ" => "chong",
    "ㄔㄡ" => "chou",
    "ㄔㄨ" => "chu",
    "ㄔㄨㄚ" => "chua",
    "ㄔㄨㄞ" => "chuai",
    "ㄔㄨㄢ" => "chuan",
    "ㄔㄨㄤ" => "chuang",
    "ㄔㄨㄟ" => "chui",
    "ㄔㄨㄣ" => "chun",
    "ㄔㄨㄛ" => "chuo",
    "ㄘ" => "ci",
    "ㄘㄨㄥ" => "cong",
    "ㄘㄡ" => "cou",
    "ㄘㄨ" => "cu",
    "ㄘㄨㄢ" => "cuan",
    "ㄘㄨㄟ" => "cui",
    "ㄘㄨㄣ" => "cun",
    "ㄘㄨㄛ" => "cuo",
    "ㄉㄚ" => "da",
    "ㄉㄞ" => "dai",
    "ㄉㄢ" => "dan",
    "ㄉㄤ" => "dang",
    "ㄉㄠ" => "dao",
    "ㄉㄜ" => "de",
    "ㄉㄥ" => "deng",
    "ㄉㄧ" => "di",
    "ㄉㄧㄢ" => "dian",
    "ㄉㄧㄤ" => "diang",
    "ㄉㄧㄠ" => "diao",
    "ㄉㄧㄝ" => "die",
    "ㄉㄧㄥ" => "ding",
    "ㄉㄧㄡ" => "diu",
    "ㄉㄨㄥ" => "dong",
    "ㄉㄡ" => "dou",
    "ㄉㄨ" => "du",
    "ㄉㄨㄢ" => "duan",
    "ㄉㄨㄟ" => "dui",
    "ㄉㄨㄣ" => "dun",
    "ㄉㄨㄛ" => "duo",
    "ㄜ" => "e",
    "ㄟ" => "ei",
    "ㄣ" => "en",
    "ㄦ" => "er",
    "ㄈㄚ" => "fa",
    "ㄈㄢ" => "fan",
    "ㄈㄤ" => "fang",
    "ㄈㄟ" => "fei",
    "ㄈㄣ" => "fen",
    "ㄈㄥ" => "feng",
    "ㄈㄛ" => "fo",
    "ㄈㄡ" => "fou",
    "ㄈㄨ" => "fu",
    "ㄍㄚ" => "ga",
    "ㄍㄞ" => "gai",
    "ㄍㄢ" => "gan",
    "ㄍㄤ" => "gang",
    "ㄍㄠ" => "gao",
    "ㄍㄜ" => "ge",
    "ㄍㄟ" => "gei",
    "ㄍㄣ" => "gen",
    "ㄍㄥ" => "geng",
    "ㄍㄨㄥ" => "gong",
    "ㄍㄡ" => "gou",
    "ㄍㄨ" => "gu",
    "ㄍㄨㄚ" => "gua",
    "ㄍㄨㄞ" => "guai",
    "ㄍㄨㄢ" => "guan",
    "ㄍㄨㄤ" => "guang",
    "ㄍㄨㄟ" => "gui",
    "ㄍㄨㄣ" => "gun",
    "ㄍㄨㄛ" => "guo",
    "ㄏㄚ" => "ha",
    "ㄏㄞ" => "hai",
    "ㄏㄢ" => "han",
    "ㄏㄤ" => "hang",
    "ㄏㄠ" => "hao",
    "ㄏㄜ" => "he",
    "ㄏㄟ" => "hei",
    "ㄏㄣ" => "hen",
    "ㄏㄥ" => "heng",
    "ㄏㄨㄥ" => "hong",
    "ㄏㄡ" => "hou",
    "ㄏㄨ" => "hu",
    "ㄏㄨㄚ" => "hua",
    "ㄏㄨㄞ" => "huai",
    "ㄏㄨㄢ" => "huan",
    "ㄏㄨㄤ" => "huang",
    "ㄏㄨㄟ" => "hui",
    "ㄏㄨㄣ" => "hun",
    "ㄏㄨㄛ" => "huo",
    "ㄐㄧ" => "ji",
    "ㄐㄧㄚ" => "jia",
    "ㄐㄧㄢ" => "jian",
    "ㄐㄧㄤ" => "jiang",
    "ㄐㄧㄠ" => "jiao",
    "ㄐㄧㄝ" => "jie",
    "ㄐㄧㄣ" => "jin",
    "ㄐㄧㄥ" => "jing",
    "ㄐㄩㄥ" => "jiong",
    "ㄐㄧㄡ" => "jiu",
    "ㄐㄩ" => "ju",
    "ㄐㄩㄢ" => "juan",
    "ㄐㄩㄝ" => "jue",
    "ㄐㄩㄣ" => "jun",
    "ㄎㄚ" => "ka",
    "ㄎㄞ" => "kai",
    "ㄎㄢ" => "kan",
    "ㄎㄤ" => "kang",
    "ㄎㄠ" => "kao",
    "ㄎㄜ" => "ke",
    "ㄎㄣ" => "ken",
    "ㄎㄥ" => "keng",
    "ㄎㄨㄥ" => "kong",
    "ㄎㄡ" => "kou",
    "ㄎㄨ" => "ku",
    "ㄎㄨㄚ" => "kua",
    "ㄎㄨㄞ" => "kuai",
    "ㄎㄨㄢ" => "kuan",
    "ㄎㄨㄤ" => "kuang",
    "ㄎㄨㄟ" => "kui",
    "ㄎㄨㄣ" => "kun",
    "ㄎㄨㄛ" => "kuo",
    "ㄌㄚ" => "la",
    "ㄌㄞ" => "lai",
    "ㄌㄢ" => "lan",
    "ㄌㄤ" => "lang",
    "ㄌㄠ" => "lao",
    "ㄌㄜ" => "le",
    "ㄌㄟ" => "lei",
    "ㄌㄥ" => "leng",
    "ㄌㄧ" => "li",
    "ㄌㄧㄢ" => "lian",
    "ㄌㄧㄤ" => "liang",
    "ㄌㄧㄠ" => "liao",
    "ㄌㄧㄝ" => "lie",
    "ㄌㄧㄣ" => "lin",
    "ㄌㄧㄥ" => "ling",
    "ㄌㄧㄡ" => "liu",
    "ㄌㄨㄥ" => "long",
    "ㄌㄡ" => "lou",
    "ㄌㄨ" => "lu",
    "ㄌㄩ" => "lü",
    "ㄌㄨㄢ" => "luan",
    "ㄌㄩㄝ" => "lüe",
    "ㄌㄨㄣ" => "lun",
    "ㄌㄨㄛ" => "luo",
    "ㄇㄚ" => "ma",
    "ㄇㄞ" => "mai",
    "ㄇㄢ" => "man",
    "ㄇㄤ" => "mang",
    "ㄇㄠ" => "mao",
    "ㄇㄟ" => "mei",
    "ㄇㄣ" => "men",
    "ㄇㄥ" => "meng",
    "ㄇㄧ" => "mi",
    "ㄇㄧㄢ" => "mian",
    "ㄇㄧㄠ" => "miao",
    "ㄇㄧㄝ" => "mie",
    "ㄇㄧㄣ" => "min",
    "ㄇㄧㄥ" => "ming",
    "ㄇㄧㄡ" => "miu",
    "ㄇㄨㄛ" => "mo",
    "ㄇㄡ" => "mou",
    "ㄇㄨ" => "mu",
    "ㄋㄚ" => "na",
    "ㄋㄞ" => "nai",
    "ㄋㄢ" => "nan",
    "ㄋㄤ" => "nang",
    "ㄋㄠ" => "nao",
    "ㄋㄛ" => "ne",
    "ㄋㄟ" => "nei",
    "ㄋㄣ" => "nen",
    "ㄋㄥ" => "neng",
    "ㄋㄧ" => "ni",
    "ㄋㄧㄚ" => "nia",
    "ㄋㄧㄢ" => "nian",
    "ㄋㄧㄤ" => "niang",
    "ㄋㄧㄠ" => "niao",
    "ㄋㄧㄝ" => "nie",
    "ㄋㄧㄣ" => "nin",
    "ㄋㄧㄥ" => "ning",
    "ㄋㄧㄡ" => "niu",
    "ㄋㄨㄥ" => "nong",
    "ㄋㄡ" => "nou",
    "ㄋㄨ" => "nu",
    "ㄋㄩ" => "nü",
    "ㄋㄨㄢ" => "nuan",
    "ㄋㄩㄝ" => "nüe",
    "ㄋㄨㄛ" => "nuo",
    "ㄋㄨㄣ" => "nun",
    "ㄡ" => "ou",
    "ㄆㄚ" => "pa",
    "ㄆㄞ" => "pai",
    "ㄆㄢ" => "pan",
    "ㄆㄤ" => "pang",
    "ㄆㄠ" => "pao",
    "ㄆㄟ" => "pei",
    "ㄆㄣ" => "pen",
    "ㄆㄥ" => "peng",
    "ㄆㄧ" => "pi",
    "ㄆㄧㄢ" => "pian",
    "ㄆㄧㄠ" => "piao",
    "ㄆㄧㄝ" => "pie",
    "ㄆㄧㄣ" => "pin",
    "ㄆㄧㄥ" => "ping",
    "ㄆㄨㄛ" => "po",
    "ㄆㄡ" => "pou",
    "ㄆㄨ" => "pu",
    "ㄑㄧ" => "qi",
    "ㄑㄧㄚ" => "qia",
    "ㄑㄧㄢ" => "qian",
    "ㄑㄧㄤ" => "qiang",
    "ㄑㄧㄠ" => "qiao",
    "ㄑㄧㄝ" => "qie",
    "ㄑㄧㄣ" => "qin",
    "ㄑㄧㄥ" => "qing",
    "ㄑㄩㄥ" => "qiong",
    "ㄑㄧㄡ" => "qiu",
    "ㄑㄩ" => "qu",
    "ㄑㄩㄢ" => "quan",
    "ㄑㄩㄝ" => "que",
    "ㄑㄩㄣ" => "qun",
    "ㄖㄢ" => "ran",
    "ㄖㄤ" => "rang",
    "ㄖㄠ" => "rao",
    "ㄖㄜ" => "re",
    "ㄖㄣ" => "ren",
    "ㄖㄥ" => "reng",
    "ㄖ" => "ri",
    "ㄖㄨㄥ" => "rong",
    "ㄖㄡ" => "rou",
    "ㄖㄨ" => "ru",
    "ㄖㄨㄢ" => "ruan",
    "ㄖㄨㄟ" => "rui",
    "ㄖㄨㄣ" => "run",
    "ㄖㄨㄛ" => "ruo",
    "ㄙㄚ" => "sa",
    "ㄙㄞ" => "sai",
    "ㄙㄢ" => "san",
    "ㄙㄤ" => "sang",
    "ㄙㄠ" => "sao",
    "ㄙㄜ" => "se",
    "ㄙㄟ" => "sei",
    "ㄙㄣ" => "sen",
    "ㄙㄥ" => "seng",
    "ㄕㄚ" => "sha",
    "ㄕㄞ" => "shai",
    "ㄕㄢ" => "shan",
    "ㄕㄤ" => "shang",
    "ㄕㄠ" => "shao",
    "ㄕㄜ" => "she",
    "ㄕㄟ" => "shei",
    "ㄕㄣ" => "shen",
    "ㄕㄥ" => "sheng",
    "ㄕ" => "shi",
    "ㄕㄡㄥ" => "shong",
    "ㄕㄡ" => "shou",
    "ㄕㄨ" => "shu",
    "ㄕㄨㄚ" => "shua",
    "ㄕㄨㄞ" => "shuai",
    "ㄕㄨㄢ" => "shuan",
    "ㄕㄨㄤ" => "shuang",
    "ㄕㄨㄟ" => "shui",
    "ㄕㄨㄣ" => "shun",
    "ㄕㄨㄛ" => "shuo",
    "ㄙ" => "si",
    "ㄙㄨㄥ" => "song",
    "ㄙㄡ" => "sou",
    "ㄙㄨ" => "su",
    "ㄙㄨㄢ" => "suan",
    "ㄙㄨㄟ" => "sui",
    "ㄙㄨㄣ" => "sun",
    "ㄙㄨㄛ" => "suo",
    "ㄊㄚ" => "ta",
    "ㄊㄞ" => "tai",
    "ㄊㄢ" => "tan",
    "ㄊㄤ" => "tang",
    "ㄊㄠ" => "tao",
    "ㄊㄜ" => "te",
    "ㄊㄥ" => "teng",
    "ㄊㄧ" => "ti",
    "ㄊㄧㄢ" => "tian",
    "ㄊㄧㄠ" => "tiao",
    "ㄊㄧㄝ" => "tie",
    "ㄊㄧㄥ" => "ting",
    "ㄊㄨㄥ" => "tong",
    "ㄊㄡ" => "tou",
    "ㄊㄨ" => "tu",
    "ㄊㄨㄢ" => "tuan",
    "ㄊㄨㄟ" => "tui",
    "ㄊㄨㄣ" => "tun",
    "ㄊㄨㄛ" => "tuo",
    "ㄨㄚ" => "wa",
    "ㄨㄞ" => "wai",
    "ㄨㄢ" => "wan",
    "ㄨㄤ" => "wang",
    "ㄨㄟ" => "wei",
    "ㄨㄣ" => "wen",
    "ㄨㄥ" => "weng",
    "ㄨㄛ" => "wo",
    "ㄨ" => "wu",
    "ㄒㄧ" => "xi",
    "ㄒㄧㄚ" => "xia",
    "ㄒㄧㄢ" => "xian",
    "ㄒㄧㄤ" => "xiang",
    "ㄒㄧㄠ" => "xiao",
    "ㄒㄧㄝ" => "xie",
    "ㄒㄧㄣ" => "xin",
    "ㄒㄧㄥ" => "xing",
    "ㄒㄩㄥ" => "xiong",
    "ㄒㄧㄡ" => "xiu",
    "ㄒㄩ" => "xu",
    "ㄒㄩㄢ" => "xuan",
    "ㄒㄩㄝ" => "xue",
    "ㄒㄩㄣ" => "xun",
    "ㄧㄚ" => "ya",
    "ㄧㄞ" => "yai",
    "ㄧㄢ" => "yan",
    "ㄧㄤ" => "yang",
    "ㄧㄠ" => "yao",
    "ㄧㄝ" => "ye",
    "ㄧ" => "yi",
    "ㄧㄣ" => "yin",
    "ㄧㄥ" => "ying",
    "ㄧㄛ" => "yo",
    "ㄩㄥ" => "yong",
    "ㄧㄡ" => "you",
    "ㄩ" => "yu",
    "ㄩㄢ" => "yuan",
    "ㄩㄝ" => "yue",
    "ㄩㄣ" => "yun",
    "ㄗㄚ" => "za",
    "ㄗㄞ" => "zai",
    "ㄗㄢ" => "zan",
    "ㄗㄤ" => "zang",
    "ㄗㄠ" => "zao",
    "ㄗㄜ" => "ze",
    "ㄗㄟ" => "zei",
    "ㄗㄣ" => "zen",
    "ㄗㄥ" => "zeng",
    "ㄓㄚ" => "zha",
    "ㄓㄞ" => "zhai",
    "ㄓㄢ" => "zhan",
    "ㄓㄤ" => "zhang",
    "ㄓㄠ" => "zhao",
    "ㄓㄜ" => "zhe",
    "ㄓㄟ" => "zhei",
    "ㄓㄣ" => "zhen",
    "ㄓㄥ" => "zheng",
    "ㄓ" => "zhi",
    "ㄓㄨㄥ" => "zhong",
    "ㄓㄡ" => "zhou",
    "ㄓㄨ" => "zhu",
    "ㄓㄨㄚ" => "zhua",
    "ㄓㄨㄞ" => "zhuai",
    "ㄓㄨㄢ" => "zhuan",
    "ㄓㄨㄤ" => "zhuang",
    "ㄓㄨㄟ" => "zhui",
    "ㄓㄨㄣ" => "zhun",
    "ㄓㄨㄛ" => "zhuo",
    "ㄗ" => "zi",
    "ㄗㄨㄥ" => "zong",
    "ㄗㄡ" => "zou",
    "ㄗㄨ" => "zu",
    "ㄗㄨㄢ" => "zuan",
    "ㄗㄨㄟ" => "zui",
    "ㄗㄨㄣ" => "zun",
    "ㄗㄨㄛ" => "zuo"
)

pinyin_to_zhuyin = Dict((value, key) for (key, value) in zhuyin_to_pinyin)

@testset "conversion tests" begin
    @testset "single char zhuyin-pinyin conversion" begin
        for (zhuyin, pinyin) in zhuyin_to_pinyin
            @test pinyin == to_pinyin(zhuyin) == to_pinyin(Zhuyin(zhuyin))
            @test zhuyin == to_zhuyin(pinyin) == to_zhuyin(Pinyin(pinyin))
        end
    end

    @testset "single char conversion w/ tone numbers" begin
        for (zhuyin, pinyin) in zhuyin_to_pinyin, tone in 1:5
            @test string(pinyin, tone) == to_pinyin(string(zhuyin, tone))
            @test string(zhuyin, tone) == to_zhuyin(string(pinyin, tone))
        end
    end

    @testset "2-char conversion w/ space separation" begin
        for (zhuyin1, pinyin1) in zhuyin_to_pinyin, (zhuyin2, pinyin2) in zhuyin_to_pinyin
            pinyin = string(pinyin1, " ", pinyin2)
            zhuyin = string(zhuyin1, " ", zhuyin2)
            @test pinyin == to_pinyin(zhuyin)
            @test zhuyin == to_zhuyin(pinyin)
        end
    end

    @testset "2-char conversions w/ no separator" begin
        test_examples = [
            ("ㄅㄧㄥ", "ㄔㄤ"),
            ("chan", "nan")
        ]

        for (char1, char2) in test_examples
            separated = string(char1, " ", char2)
            non_separated = string(char1, char2)
            @test replace(to_pinyin(separated), " " => "") == to_pinyin(non_separated)
            @test replace(to_zhuyin(separated), " " => "") == to_zhuyin(non_separated)
        end
    end

    @testset "special cases for u:" begin
        @test to_zhuyin("lu:") == to_zhuyin("lü")
        @test to_zhuyin("lu:e") == to_zhuyin("lüe")
    end

    @testset "preserve partial sequences" begin
        partials = ["ch", "sh", "co", "ts", "x", "y"]
        for partial_string in partials
            @test to_zhuyin(partial_string) == partial_string
        end

        @test to_zhuyin("ch sh co x y w") == "ch sh co x y w"
        @test to_zhuyin("chshcoxyw") == "chshcoxyw"
    end

end
